@@ -1,12 +1,21 @@
 import re
 
 from lxml import etree
+import six
+if six.PY3:
+    unicode = str
 
 # parsing functions
 
 
 def is_xnat_error(message):
-    return message.startswith('<!DOCTYPE') or message.startswith('<html>')
+    a = ['<!DOCTYPE', '<html>']
+    try:
+        return message.startswith(a[0]) or message.startswith(a[1])
+    except TypeError:
+        if isinstance(message, bytes):
+            a = [bytes(e, 'utf-8') for e in a]
+        return message.startswith(a[0]) or message.startswith(a[1])
 
 
 def parse_error_message(message):
@@ -56,6 +65,9 @@ def parse_put_error_message(message):
 
 def catch_error(msg_or_exception, full_response=None):
 
+    if isinstance(msg_or_exception, bytes):
+        msg_or_exception = msg_or_exception.decode()
+
     # handle errors returned by the xnat server
     if isinstance(msg_or_exception, (str, unicode)):
         # parse the message
