@@ -6,36 +6,39 @@ import sys
 import re
 import shutil
 import tempfile
-import mimetypes
 import zipfile
 import time
-import urllib
 import codecs
 from fnmatch import fnmatch
 from itertools import islice
-
-import json
 from lxml import etree
-
 from .uriutil import join_uri, translate_uri, uri_segment
 from .uriutil import uri_last, uri_nextlast
 from .uriutil import uri_parent, uri_grandparent
 from .uriutil import uri_shape
 from .uriutil import file_path
-
 from .jsonutil import JsonTable, get_selection
 from .pathutil import find_files, ensure_dir_exists
 from .attributes import EAttrs
 from .search import rpn_contraints, query_from_xml
 from .errors import is_xnat_error, parse_put_error_message
 from .errors import DataError, ProgrammingError, catch_error
-from .cache import md5name
 from .provenance import Provenance
 # from .pipelines import Pipelines
 from . import schema
 from . import httputil
 from . import downloadutils
-
+from . import derivatives
+import types
+import pkgutil
+import inspect
+import six
+from six import string_types, add_metaclass
+if six.PY2:
+    from urllib import quote, unquote  # Python 2.X
+elif six.PY3:
+    from urllib.parse import quote, unquote
+    unicode = str
 
 DEBUG = False
 
@@ -71,7 +74,6 @@ def get_element_from_collection(rsc_name):
 def get_collection_from_element(rsc_name):
 
     def getter(self, id_filter='*'):
-
         Collection = globals()[rsc_name.title()]
         return Collection(join_uri(self._uri, rsc_name),
                           self._intf, id_filter
@@ -93,6 +95,7 @@ def get_collection_from_collection(rsc_name):
 
 class ElementType(type):
     def __new__(cls, name, bases, dct):
+
         rsc_name = name.lower() + 's' \
             if name.lower() in schema.resources_singular \
             else name.lower()
@@ -124,7 +127,28 @@ class CollectionType(type):
     def __init__(cls, name, bases, dct):
         super(CollectionType, cls).__init__(name, bases, dct)
 
+
 # generic classes
+def __get_modules__(m):
+    modules = []
+    prefix = m.__name__ + '.'
+    for importer, modname, ispkg in pkgutil.iter_modules(m.__path__, prefix):
+        module = __import__(modname, fromlist='dummy')
+        if not ispkg:
+            modules.append(module)
+        else:
+            modules.extend(__get_modules__(module))
+    return modules
+
+
+def __find_all_functions__(m):
+    functions = {}
+    modules = __get_modules__(m)
+    for m in modules:
+        for name, obj in inspect.getmembers(m):
+            if inspect.isfunction(obj):
+                functions.setdefault(m, []).append(obj)
+    return functions
 
 
 class EObject(object):
@@ -141,12 +165,27 @@ class EObject(object):
             interface: :class:`Interface`
                 Main interface reference.
         """
-        self._uri = urllib.quote(translate_uri(uri))
-        self._urn = urllib.unquote(uri_last(self._uri))
+
+        self._uri = quote(translate_uri(uri))
+        self._urn = unquote(uri_last(self._uri))
         self._urt = uri_nextlast(self._uri)
         self._intf = interface
         self.attrs = EAttrs(self)
 
+        functions = __find_all_functions__(derivatives)
+
+        for m, mod_functions in functions.items():
+            is_resource = False
+            if (hasattr(m, 'XNAT_RESOURCE_NAME') and
+                    self._urn == m.XNAT_RESOURCE_NAME) or \
+                    (hasattr(m, 'XNAT_RESOURCE_NAMES') and
+                        self._urn in m.XNAT_RESOURCE_NAMES):
+                is_resource = True
+
+            if is_resource:
+                for f in mod_functions:
+                    setattr(self, f.__name__, types.MethodType(f, self))
+
     def __getstate__(self):
         return {
             'uri': self._uri,
@@ -158,7 +197,7 @@ class EObject(object):
 
     def __repr__(self):
         return '<%s Object> %s' % (self.__class__.__name__,
-                                   urllib.unquote(uri_last(self._uri))
+                                   unquote(uri_last(self._uri))
                                    )
 
     def _getcell(self, col):
@@ -191,7 +230,7 @@ class EObject(object):
         if filters:
             get_id += '&' + \
                 '&'.join('%s=%s' % (item[0], item[1])
-                         if isinstance(item[1], basestring)
+                         if isinstance(item[1], string_types)
                          else '%s=%s' % (item[0],
                                          ','.join([val for val in item[1]])
                                          )
@@ -258,26 +297,32 @@ class EObject(object):
                To set up additional variables for the element at its
                creation it is possible to use shortcuts defined in the
                XNAT REST documentation or xpath in the schema:
-                   - element.create(ID='theid')
-                   - subject.create(**{'xnat:subjectData/ID':'theid'})
+
+                   - `element.create(ID='theid')`
+                   - `subject.create(**{'xnat:subjectData/ID':'theid'})`
 
 
             Parameters
             ----------
             params: keywords
+
                 - Specify the datatype of the element resource and of
                   any ancestor that may need to be created. The
                   keywords correspond to the levels in the REST
                   hierarchy, see Interface.inspect.architecture()
+
                 - If an element is created with no specified type:
+
                       - if its name matches a naming convention, this type
                         will be used
                       - else a default type is defined in the schema module
+
                 - To give the ID the same value as the label use
                   use_label=True e.g element.create(use_label=True)
 
             Examples
             --------
+
                 >>> interface.select('/project/PROJECT/subject'
                                      '/SUBJECT/experiment/EXP/scan/SCAN'
                             ).create(experiments='xnat:mrSessionData',
@@ -286,9 +331,9 @@ class EObject(object):
 
             See Also
             --------
-            EObject.id
-            EObject.label
-            EObject.datatype
+            :func:`EObject.id`
+            :func:`EObject.label`
+            :func:`EObject.datatype`
         """
         if 'xml' in params and os.path.exists(params.get('xml')):
 
@@ -300,14 +345,15 @@ class EObject(object):
                 doc_tree = etree.fromstring(doc)
                 doc_tree.xpath('//*')[0].set('label', uri_last(self._uri))
                 doc = etree.tostring(doc_tree)
-            except:
+            except Exception:
                 pass
 
             body, content_type = httputil.file_message(
-                doc, 'text/xml', 'data.xml', 'data.xml')
+                doc.decode(), 'text/xml', 'data.xml', 'data.xml')
 
             _uri = self._uri
-            if ('allowDataDeletion' in params and params.get('allowDataDeletion') is False):
+            if ('allowDataDeletion' in params and
+                    params.get('allowDataDeletion') is False):
                 _uri += '?allowDataDeletion=false'
             else:
                 _uri += '?allowDataDeletion=true'
@@ -315,8 +361,7 @@ class EObject(object):
             self._intf._exec(_uri,
                              method='PUT',
                              body=body,
-                             headers={'content-type': content_type}
-                             )
+                             headers={'content-type': content_type})
 
             return self
 
@@ -376,10 +421,12 @@ class EObject(object):
             print('PUT', create_uri)
 
         if 'params' in params and 'event_reason' in params['params']:
-            if DEBUG: print('Have event_reason')
+            if DEBUG:
+                print('Found event_reason')
             output = self._intf._exec(create_uri, 'PUT', **params)
         else:
-            if DEBUG: print('Not have event_reason')
+            if DEBUG:
+                print('event_reason not found')
             output = self._intf._exec(create_uri, 'PUT')
 
         if is_xnat_error(output):
@@ -490,6 +537,7 @@ class CObject(object):
 
         A collection resource is a list of element resources. There is
         however several ways to obtain such a list:
+
             - a collection URI e.g. /REST/projects
             - a list of element URIs
             - a list of collections
@@ -499,6 +547,7 @@ class CObject(object):
             - a list a collection objects
 
         Collections objects built in different ways share the same behavior:
+
             - they behave as iterators, which enables a lazy access to
               the data
             - they always yield EObjects
@@ -551,12 +600,12 @@ class CObject(object):
         self._filters = filters
         self._nested = nested
 
-        if isinstance(cbase, basestring):
+        if isinstance(cbase, string_types):
             self._ctype = 'cobjectcuri'
         elif isinstance(cbase, CObject):
             self._ctype = 'cobjectcobject'
         elif isinstance(cbase, list) and cbase:
-            if isinstance(cbase[0], basestring):
+            if isinstance(cbase[0], string_types):
                 self._ctype = 'cobjecteuris'
             if isinstance(cbase[0], EObject):
                 self._ctype = 'cobjecteobjects'
@@ -573,7 +622,7 @@ class CObject(object):
     def _call(self, columns):
         try:
             uri = translate_uri(self._cbase)
-            uri = urllib.quote(uri)
+            uri = quote(uri)
 
             request_shape = uri_shape(
                 '%s/0' % uri.split(self._intf._get_entry_point(), 1)[1])
@@ -585,7 +634,6 @@ class CObject(object):
                 self._intf.inspect._tick == 0 and\
                 self._intf.inspect._auto
 
-
             columns += ['xsiType']
 
             query_string = '?format=json&columns=%s' % ','.join(columns)
@@ -600,10 +648,9 @@ class CObject(object):
                     )
 
             if DEBUG:
-                print uri + query_string
+                print(uri + query_string)
             jtable = self._intf._get_json(uri + query_string)
 
-
             _type = uri.split('/')[-1]
             self._learn_from_table(_type, jtable, None)
 
@@ -619,17 +666,17 @@ class CObject(object):
         for element in jtable:
             xsitype = element.get('xsiType')
             if xsitype:
-                #Only some of the xnat elements return xsiType like we asked them to.
-                #Projects and Subjects are two known offenders so we cannot update our
-                #knowledge of them.
-                uri = element.get('URI').split(self._intf._get_entry_point(), 1)[1]
+                # Only some of the xnat elements return xsiType like we asked
+                # them to.Projects and Subjects are two known offenders so we
+                #  cannot update our knowledge of them.
+                uri = element.get('URI').split(self._intf._get_entry_point(),
+                                               1)[1]
                 uri = uri.replace(uri.split('/')[-2], _type)
                 shape = uri_shape(uri)
                 request_knowledge[shape] = xsitype
 
         self._intf._struct.update(request_knowledge)
 
-
     def __iter__(self):
         if self._ctype == 'cobjectcuri':
             if self._id_header == 'ID':
@@ -641,7 +688,7 @@ class CObject(object):
 
             for res in self._call([id_header] + self._columns):
                 try:
-                    eid = urllib.unquote(res[id_header])
+                    eid = unquote(res[id_header])
                     if fnmatch(eid, self._pattern):
                         klass_name = uri_last(self._cbase
                                               ).rstrip('s').title()
@@ -820,7 +867,7 @@ class CObject(object):
                   instead of a list.
         """
         if not args:
-            return [urllib.unquote(uri_last(eobj._uri)) for eobj in self]
+            return [unquote(uri_last(eobj._uri)) for eobj in self]
         else:
             entries = []
 
@@ -829,10 +876,10 @@ class CObject(object):
                 for arg in args:
                     if arg == 'id':
                         self._id_header = 'ID'
-                        entry += (urllib.unquote(uri_last(eobj._uri)),)
+                        entry += (unquote(uri_last(eobj._uri)),)
                     elif arg == 'label':
                         self._id_header = 'label'
-                        entry += (urllib.unquote(uri_last(eobj._uri)),)
+                        entry += (unquote(uri_last(eobj._uri)),)
                     else:
                         entry += (eobj,)
 
@@ -897,7 +944,7 @@ class CObject(object):
 
             See Also
             --------
-            search.Search()
+            :func:`search.Search`
         """
 
         if isinstance(constraints, (str, unicode)):
@@ -981,8 +1028,8 @@ class CObject(object):
 # specialized classes
 
 
+@add_metaclass(ElementType)
 class Project(EObject):
-    __metaclass__ = ElementType
 
     def __init__(self, uri, interface):
         """
@@ -1106,9 +1153,9 @@ class Project(EObject):
             string : owner | member | collaborator
 
         """
-        return JsonTable(self._intf._get_json(join_uri(self._uri, 'users'))
+        return ''.join(JsonTable(self._intf._get_json(join_uri(self._uri, 'users'))
                          ).where(login=login
-                                 )['displayname'].lower().rstrip('s')
+                                 )['displayname']).lower().rstrip('s')
 
     def add_user(self, login, role='member'):
         """ Adds a user to the project. The user must already exist on
@@ -1206,6 +1253,7 @@ class Project(EObject):
 
         """
         tree = lxml.etree.fromstring(self.get())
+
         update = False
 
         for protocol, value in custom_variables.items():
@@ -1230,6 +1278,7 @@ class Project(EObject):
                     )
                 protocol_element.append(definitions_element)
 
+
             for group, fields in value.items():
                 try:
                     group_element = definitions_element.xpath(
@@ -1239,8 +1288,10 @@ class Project(EObject):
                     fields_element = group_element.xpath(
                         "xnat:fields",
                         namespaces=tree.nsmap).pop()
+
                 except IndexError:
                     update = True
+
                     group_element = lxml.etree.Element(
                         lxml.etree.QName(tree.nsmap['xnat'], 'definition'),
                         nsmap=tree.nsmap
@@ -1280,7 +1331,7 @@ class Project(EObject):
                         update = True
         if update:
             body, content_type = httputil.file_message(
-                lxml.etree.tostring(tree),
+                lxml.etree.tostring(tree).decode('utf-8'),
                 'text/xml',
                 'cust.xml',
                 'cust.xml'
@@ -1289,7 +1340,7 @@ class Project(EObject):
             uri = self._uri
             if allow_data_deletion:
                 uri = self._uri + '?allowDataDeletion=true'
-            self._intf._exec(uri, method='PUT', body=body,
+            self._intf._exec(uri, method='PUT', body=str(body),
                              headers={'content-type': content_type})
 
     def get_custom_variables(self):
@@ -1323,9 +1374,27 @@ class Project(EObject):
 
         return custom_variables
 
+    def aliases(self):
+        """Returns the aliases for this project.
 
+           Returns
+           -------
+           List of aliases
+        """
+
+        uri = '/data/projects'
+        options = {'columns': 'alias', 'format':'csv'}
+        data = self._intf.get(uri, params=options).text
+        from .jsonutil import csv_to_json
+        data = csv_to_json(data)
+
+        # parse the results
+        return [item['alias'] for item in data if item['alias'] and \
+            item['ID'] == self._urn]
+
+
+@add_metaclass(ElementType)
 class Subject(EObject):
-    __metaclass__ = ElementType
 
     def datatype(self):
         return 'xnat:subjectData'
@@ -1361,8 +1430,8 @@ class Subject(EObject):
         self._intf._exec(join_uri(self._uri, 'projects', project), 'DELETE')
 
 
+@add_metaclass(ElementType)
 class Experiment(EObject):
-    __metaclass__ = ElementType
 
     def shares(self, id_filter='*'):
         """ Returns the projects sharing this experiment.
@@ -1433,11 +1502,11 @@ class Experiment(EObject):
 
             options = '?' + '&'.join(options)
 
-            self._intf._exec(self._uri + options, 'PUT')
+            self._intf._exec(self._uri + options, 'PUT', body=[])
 
 
+@add_metaclass(ElementType)
 class Assessor(EObject):
-    __metaclass__ = ElementType
 
     def __init__(self, uri, interface):
         EObject.__init__(self, uri, interface)
@@ -1491,8 +1560,8 @@ class Assessor(EObject):
         return self.xpath('//xnat:addParam/attribute::*')
 
 
+@add_metaclass(ElementType)
 class Reconstruction(EObject):
-    __metaclass__ = ElementType
 
     def __init__(self, uri, interface):
         EObject.__init__(self, uri, interface)
@@ -1504,9 +1573,8 @@ class Reconstruction(EObject):
                 or 'xnat:reconstructedImageData'
                 )
 
-
+@add_metaclass(ElementType)
 class Scan(EObject):
-    __metaclass__ = ElementType
 
     def set_param(self, key, value):
         self.attrs.set('%s/parameters/addParam[name=%s]/addField'
@@ -1524,9 +1592,8 @@ class Scan(EObject):
     def params(self):
         return self.xpath('//xnat:addParam/attribute::*')
 
-
+@add_metaclass(ElementType)
 class Resource(EObject):
-    __metaclass__ = ElementType
 
     def get(self, dest_dir, extract=False):
         """ Downloads all the files within a resource.
@@ -1570,13 +1637,13 @@ class Resource(EObject):
                             #flush the buffer every once in a while.
                             f.flush()
                 f.flush()  # and one last flush.
-            except Exception, e:
+            except Exception as e:
                 sys.stderr.write(e)
             finally:
                 response.close()
 
         if DEBUG:
-            print zip_location
+            print(zip_location)
 
         fzip = zipfile.ZipFile(zip_location, 'r')
         fzip.extractall(path=dest_dir)
@@ -1672,7 +1739,7 @@ class Resource(EObject):
             #use compression if avaiable.
             fzip = zipfile.ZipFile(zip_name, 'w', zipfile.ZIP_DEFLATED)
         except RuntimeError:
-            print "Zip compression not supported for uploading files."
+            print("Zip compression not supported for uploading files.")
             fzip = zipfile.ZipFile(zip_name, 'w')
 
         for src in sources:
@@ -1740,29 +1807,43 @@ class Resource(EObject):
                 or 'xnat:abstractResource'
                 )
 
+    def attributes(self):
+        """ Files attributes include:
+                - URI
+                - Name
+                - Size in bytes
+                - path (relative to the parent resource)
+                - tags
+                - format
+                - content
+            Returns
+            -------
+            dict : a dictionary with the resource attributes
+        """
+
+        return self._getcells(['URI', 'Name', 'Size', 'path',
+                               'tags', 'format', 'content'])
+
 
 class In_Resource(Resource):
-    __metaclass__ = ElementType
 
     def parent(self):
         uri = uri_grandparent(self._uri)
         Klass = globals()[uri.split('/')[-3].title().rsplit('s', 1)[0]]
         return Klass(uri_parent(uri), self._intf)
 
-
+@add_metaclass(ElementType)
 class Out_Resource(Resource):
-    __metaclass__ = ElementType
 
     def parent(self):
         uri = uri_grandparent(self._uri)
         Klass = globals()[uri.split('/')[-3].title().rsplit('s', 1)[0]]
         return Klass(uri_parent(uri), self._intf)
 
-
+@add_metaclass(ElementType)
 class File(EObject):
     """ EObject for files stored in XNAT.
     """
-    __metaclass__ = ElementType
 
     def __init__(self, uri, interface):
         """
@@ -1794,31 +1875,24 @@ class File(EObject):
 
             Returns
             -------
-            dict : a dictionnary with the file attributes
+            dict : a dictionary with the file attributes
         """
 
         return self._getcells(['URI', 'Name', 'Size', 'path',
                                'file_tags', 'file_format', 'file_content'])
 
-    def get(self, dest=None, force_default=False):
+    def get(self, dest=None):
         """ Downloads the file.
 
             Parameters
             ----------
             dest: string | None
+
                 - If dest is None, then the user's Downloads directory is used
                     as the default download location.
                 - Else the file is downloaded at the requested location.
                     Path should include the file name.
                     eg: /path/to/file.txt
-            force_default: boolean
-                - Depricated as of 1.0.0.0
-                - Has no effect if the file is downloaded for the first time
-                - If the file was previously download with a custom path,
-                  calling get() will remember the custom location unless:
-                      - another custom location is set in dest
-                      - force_default is set to True and the file will be
-                        moved to the cache
 
             Returns
             -------
@@ -1834,11 +1908,11 @@ class File(EObject):
             dest = os.path.join(os.path.expanduser("~"), 'Downloads', self.id())
             if not ensure_dir_exists(os.path.dirname(dest)):
                 if DEBUG:
-                    print "File.get: failed to create dir"
+                    print("File.get: failed to create dir")
                 raise DataError('Cannot create dir for file: %s' % (dest))
 
         if DEBUG:
-            print "get_file:", dest
+            print(("get_file:", dest))
 
         with open(dest, 'wb') as f:
             response = self._intf.get(self._uri, stream=True)
@@ -1852,7 +1926,7 @@ class File(EObject):
                             #flush the buffer every once in a while.
                             f.flush()
                 f.flush()  # and one last flush.
-            except Exception, e:
+            except Exception as e:
                 sys.stderr.write(e)
             finally:
                 response.close()
@@ -1919,11 +1993,11 @@ class File(EObject):
             else:
                 path = self._uri.split('/')[-1]
                 name = path
-        except:
+        except Exception:
             path = self._uri.split('/')[-1]
             name = path
 
-        self._absuri = urllib.unquote(
+        self._absuri = unquote(
             re.sub('resources/.*?/',
                    'resources/%s/' % resource_id, self._uri)
             )
@@ -1945,8 +2019,8 @@ class File(EObject):
             self._absuri = self._absuri.split('?')[0]
 
         if DEBUG:
-            print 'INSERT FILE', os.path.exists(src)
-            print "URI is: " + self._absuri
+            print(('INSERT FILE', os.path.exists(src)))
+            print("URI is: " + self._absuri)
 
         response = None
         if isFile:
@@ -1964,11 +2038,11 @@ class File(EObject):
                 print(response.get("status"))
 
             catch_error(response.content, '''pyxnat.file.put failure:
-    URI: {response.url}
-    status code: {response.status_code}
-    headers: {response.headers}
-    content: {response.content}
-'''.format(response=response))
+                                             URI: {response.url}
+                                             status code: {response.status_code}
+                                             headers: {response.headers}
+                                             content: {response.content}
+                                          '''.format(response=response))
 
     insert = put
     create = put
@@ -2018,20 +2092,23 @@ class File(EObject):
         return info['last-modified']
 
 
+@add_metaclass(ElementType)
 class In_File(File):
-    __metaclass__ = ElementType
+    pass
 
 
+@add_metaclass(ElementType)
 class Out_File(File):
-    __metaclass__ = ElementType
+    pass
 
 
+@add_metaclass(CollectionType)
 class Projects(CObject):
-    __metaclass__ = CollectionType
+    pass
 
 
+@add_metaclass(CollectionType)
 class Subjects(CObject):
-    __metaclass__ = CollectionType
 
     def sharing(self, projects=[]):
         return Subjects([eobj for eobj in self
@@ -2049,8 +2126,8 @@ class Subjects(CObject):
             eobj.unshare(project)
 
 
+@add_metaclass(CollectionType)
 class Experiments(CObject):
-    __metaclass__ = CollectionType
 
     def sharing(self, projects=[]):
         return Experiments([eobj for eobj in self
@@ -2068,8 +2145,8 @@ class Experiments(CObject):
             eobj.unshare(project)
 
 
+@add_metaclass(CollectionType)
 class Assessors(CObject):
-    __metaclass__ = CollectionType
 
     def sharing(self, projects=[]):
         return Assessors([eobj for eobj in self
@@ -2095,8 +2172,8 @@ class Assessors(CObject):
                                       extract, safe, removeZip)
 
 
+@add_metaclass(CollectionType)
 class Reconstructions(CObject):
-    __metaclass__ = CollectionType
 
     def download(self, dest_dir, type="ALL",
                  name=None, extract=False, safe=False, removeZip=False):
@@ -2107,8 +2184,8 @@ class Reconstructions(CObject):
                                       extract, safe, removeZip)
 
 
+@add_metaclass(CollectionType)
 class Scans(CObject):
-    __metaclass__ = CollectionType
 
     def download(self, dest_dir, type="ALL",
                  name=None, extract=False, safe=False, removeZip=False):
@@ -2120,28 +2197,34 @@ class Scans(CObject):
                                       extract, safe, removeZip)
 
 
+@add_metaclass(CollectionType)
 class Resources(CObject):
-    __metaclass__ = CollectionType
+    pass
 
 
+@add_metaclass(CollectionType)
 class In_Resources(Resources):
-    __metaclass__ = CollectionType
+    pass
 
 
+@add_metaclass(CollectionType)
 class Out_Resources(Resources):
-    __metaclass__ = CollectionType
+    pass
 
 
+@add_metaclass(CollectionType)
 class Files(CObject):
-    __metaclass__ = CollectionType
+    pass
 
 
+@add_metaclass(CollectionType)
 class In_Files(Files):
-    __metaclass__ = CollectionType
+    pass
 
 
+@add_metaclass(CollectionType)
 class Out_Files(Files):
-    __metaclass__ = CollectionType
+    pass
 
 ## Utility functions for downloading and extracting zip archives
 
