@@ -1,15 +1,16 @@
-import os
-import re
-import glob
 import csv
 import difflib
 try:
     from StringIO import StringIO
 except ImportError:
     from io import StringIO
+try:
+    unicode
+except NameError:
+    unicode = str
+from six import string_types
 
 from lxml import etree
-import json
 
 from .jsonutil import JsonTable, get_column, get_where, get_selection
 from .errors import is_xnat_error, catch_error
@@ -126,7 +127,7 @@ def build_search_document(root_element_name, columns, criteria_set,
 def build_criteria_set(container_node, criteria_set):
 
     for criteria in criteria_set:
-        if isinstance(criteria, basestring):
+        if isinstance(criteria, string_types):
             container_node.set('method', criteria)
 
         if isinstance(criteria, (list)):
@@ -360,7 +361,7 @@ class SearchManager(object):
 
             See Also
             --------
-            Search.where
+            :func:`Search.where`
         """
         self._save_search(row, columns, constraints,
                           name, description, sharing)
@@ -428,9 +429,9 @@ class SearchManager(object):
             '%s/search/saved/%s/results?format=csv' % (self._intf._entry,
                                                        search_id), 'GET')
 
-        results = csv.reader(StringIO(content), delimiter=',', quotechar='"')
+        results = csv.reader(StringIO(content.decode('utf-8')), delimiter=',', quotechar='"')
 
-        headers = results.next()
+        headers = next(results)
 
         return JsonTable([dict(zip(headers, res))
                           for res in results
@@ -465,7 +466,7 @@ class SearchManager(object):
             Parameters
             ----------
             name: string
-                Name under which the template is save in XNAT. A 'template_' is
+                Name under which the template is save in XNAT. A template is
                 prepended to the name so that it appear clearly as a template
                 on the web interface.
             row: string
@@ -536,6 +537,8 @@ class SearchManager(object):
 
     def use_template(self, name, values):
         """
+            Performs a search query using a previously saved template.
+
             Parameters
             ----------
             name: string
@@ -705,8 +708,8 @@ class Search(object):
         if is_xnat_error(content):
             catch_error(content)
 
-        results = csv.reader(StringIO(content), delimiter=',', quotechar='"')
-        headers = results.next()
+        results = csv.reader(StringIO(content.decode('utf-8')), delimiter=',', quotechar='"')
+        headers = next(results)
 
         headers_of_interest = []
 
@@ -729,4 +732,3 @@ class Search(object):
 
     def all(self):
         return self.where([(self._row + '/ID', 'LIKE', '%'), 'AND'])
-
