@@ -1,4 +1,4 @@
-from .. import xpass
+from pyxnat import xpass
 
 def test_find_plus_line():
     print("Testing find_plus_line")
@@ -30,4 +30,3 @@ def test_parse_xnat_pass():
     assert(xpass.parse_xnat_pass([lineWithoutUser]) == None)
     assert(xpass.parse_xnat_pass([lineWithoutHost]) == None)
     assert(xpass.parse_xnat_pass([lineWithoutPass]) == None)
-
