@@ -1,22 +1,29 @@
-import os
-from .. import Interface
+import os.path as op
+from pyxnat import Interface
+from . import skip_if_no_network
+from nose import SkipTest
 
-central = Interface(config=os.path.join(os.path.dirname(os.path.abspath(__file__)), 'central.cfg'))
+central = Interface(config=op.join(op.dirname(op.abspath(__file__)), 'central.cfg'))
 
-def test_xpath_checkout():
+@skip_if_no_network
+def test_001_xpath_checkout():
     central.xpath.checkout(subjects=['OAS1_0001', 'OAS1_0002'])
     assert 'OAS1_0001' in central.xpath.subjects() and \
         'OAS1_0002' in central.xpath.subjects()
 
+@skip_if_no_network
 def test_elements():
     assert 'fs:region' in central.xpath.elements()
 
+@skip_if_no_network
 def test_keys():
     assert 'ID' in central.xpath.keys()
 
+@skip_if_no_network
 def test_values():
     assert 'OAS1_0002' in central.xpath.values('ID')
 
+@skip_if_no_network
 def test_element_attrs():
     assert isinstance(central.xpath.element_attrs('fs:region'), list)
 
@@ -25,4 +32,3 @@ def test_element_attrs():
 
     assert 'Left-Putamen' in \
         central.xpath.element_values('fs:region', 'name')
-
